@@ -13,7 +13,7 @@
 
 """
 from __future__ import annotations
-__version__ = '2.38.1'
+__version__ = '2.52.0'
 
 #: Chemical engineering plant cost index (defaults to 567.5 at 2017).
 CE: float = 567.5 
@@ -21,9 +21,11 @@ CE: float = 567.5
 #: User-defined impact indicators and their units of measure.
 impact_indicators: dict[str, str] = {}
 
-#: Price of stream utilities [USD/kg] which are defined as 
+#: Price of stream utilities/fees/credits [USD/kg] which are defined as 
 #: inlets and outlets to unit operations.
-stream_utility_prices: dict[str, float] = {}
+stream_prices: dict[str, float] = {}
+
+stream_utility_prices = stream_prices # For backwards compatibility
 
 #: Defined allocation property and basis pairs for LCA.
 allocation_properties: dict[str, str] = {}
@@ -47,7 +49,7 @@ else:
 
 import thermosteam
 from thermosteam import *
-from ._preferences import preferences
+import thermosteam.units_of_measure # Import the units_of_measure module to override the units_of_measure decorator.
 from ._heat_utility import UtilityAgent, HeatUtility
 from ._power_utility import PowerUtility
 from . import plots
@@ -62,11 +64,11 @@ from .process_tools import *
 from . import _tea
 from ._tea import *
 from . import utils
-from . import _temporary_connection
-from ._temporary_connection import *
 from . import units
 from .units import *
 from ._facility import Facility
+from . import _module
+from ._module import *
 from . import facilities
 from .facilities import *
 from . import wastewater
@@ -80,16 +82,29 @@ from . import _settings
 __all__ = (
     'Unit', 'PowerUtility', 'UtilityAgent', 'HeatUtility', 'Facility',
     'utils', 'units', 'facilities', 'wastewater', 'evaluation', 'Chemical', 'Chemicals', 'Stream',
-    'MultiStream', 'settings', 'exceptions', 'report',
+    'MultiStream', 'settings', 'exceptions', 'report', 'units_of_measure',
     'process_tools', 'preferences', *_system.__all__, *_flowsheet.__all__, 
     *_tea.__all__, *units.__all__, *facilities.__all__, *wastewater.__all__,
-    *evaluation.__all__, *process_tools.__all__, *_temporary_connection.__all__,
+    *evaluation.__all__, *process_tools.__all__, *_module.__all__,
 )
 
-def nbtutorial():
+def nbtutorial(dark=False):
+    main_flowsheet.clear()
     preferences.reset()
-    preferences.light_mode(bg='#ffffffaa')
+    if dark: 
+        preferences.dark_mode(bg='#111111')
+    else:
+        preferences.light_mode(bg='#ffffffaa')
     preferences.tooltips_full_results = False
     preferences.graphviz_format = 'html'
+    preferences.show_all_streams = True
     from warnings import filterwarnings
     filterwarnings('ignore')
+    
+    import pandas as pd
+    from IPython.display import display
+
+    def display_table_as_html(series):
+        return display(series.to_frame())
+
+    pd.Series._ipython_display_ = display_table_as_html
