@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2024, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -9,10 +9,10 @@
 """
 from __future__ import annotations
 from thermosteam.units_of_measure import (
-    convert, DisplayUnits, AbsoluteUnitsOfMeasure, get_dimensionality,
+    convert, DisplayUnits, UnitsOfMeasure, get_dimensionality,
     heat_utility_units_of_measure
 )
-from thermosteam.utils import unregistered, units_of_measure
+from thermosteam.utils import unregistered, define_units_of_measure
 from thermosteam import Thermo, Stream, ThermalCondition, settings
 from .exceptions import DimensionError
 from math import copysign
@@ -26,9 +26,9 @@ __all__ = ('HeatUtility', 'UtilityAgent')
 # ^This table was made using data from Busche, 1995
 # Entry temperature conditions of coolants taken from Table 12.1 in Warren, 2016
 
-mol_basis_units = AbsoluteUnitsOfMeasure('kmol')
-mass_basis_units = AbsoluteUnitsOfMeasure('kg')
-energy_basis_units = AbsoluteUnitsOfMeasure('kJ')
+mol_basis_units = UnitsOfMeasure('kmol')
+mass_basis_units = UnitsOfMeasure('kg')
+energy_basis_units = UnitsOfMeasure('kJ')
 
 # %% Utility agents
 
@@ -62,17 +62,23 @@ class UtilityAgent(Stream):
         the maximum temperature. If utility agent cools down, `T_limit` is
         the minimum temperature. 
     heat_transfer_price :
-        Price of transferred heat [USD/kJ]. Defautls to 1.
+        Price of transferred heat [USD/kJ]. Defaults to 1.
     regeneration_price :
         Price of regenerating the fluid for reuse [USD/kmol]. Defaults to 0.
     heat_transfer_efficiency :
         Fraction of heat transferred accounting for losses to the environment (must be between 0 to 1). Defaults to 1.
+    isfuel :
+        Whether to burn the agent as a isfuel for heat.
+    dT :
+        Minimum temperature change between inlet and outlet utility. A positive value
+        prevents near infinite flows when utility agents use sensible heats.
     **chemical_flows : float
         ID - flow pairs.
         
     """
     __slots__ = ('T_limit', '_heat_transfer_price', 'utility_stream_dump',
-                 '_regeneration_price', 'heat_transfer_efficiency')
+                 '_regeneration_price', 'heat_transfer_efficiency', 'isfuel',
+                 'dT')
     def __init__(self, 
                  ID: Optional[str]='',
                  phase: Optional[str]='l',
@@ -83,11 +89,13 @@ class UtilityAgent(Stream):
                  T_limit: Optional[float]=None,
                  heat_transfer_price: float=0.,
                  regeneration_price: float=0.,
-                 heat_transfer_efficiency: float=1.,                  
+                 heat_transfer_efficiency: float=1.,
+                 isfuel: bool=False,
+                 dT: Optional[float]=0,
                  **chemical_flows: float):
         self._thermal_condition = ThermalCondition(T, P)
         thermo = self._load_thermo(thermo)
-        self._init_indexer((), phase, thermo.chemicals, chemical_flows)
+        self._init_indexer(None, phase, thermo.chemicals, chemical_flows)
         if units is not None:
             name, factor = self._get_flow_name_and_factor(units)
             flow = getattr(self, name)
@@ -101,6 +109,8 @@ class UtilityAgent(Stream):
         self.heat_transfer_price = heat_transfer_price
         self.regeneration_price = regeneration_price
         self.heat_transfer_efficiency = heat_transfer_efficiency
+        self.isfuel = isfuel
+        self.dT = dT
         self.utility_stream_dump = []
     
     def _get_property(self, name, flow=False, nophase=False, T=None, P=None):
@@ -115,7 +125,7 @@ class UtilityAgent(Stream):
         if nophase:
             literal = (T, P)
         else:
-            phase = imol._phase._phase
+            phase = imol._phase
             literal = (phase, T, P)
         key = (name, literal)
         if key in property_cache: return property_cache[key]
@@ -232,7 +242,7 @@ class UtilityAgent(Stream):
 
 # %%
 
-@units_of_measure(heat_utility_units_of_measure)
+@define_units_of_measure(heat_utility_units_of_measure)
 class HeatUtility:
     """
     Create an HeatUtility object that can choose a utility stream and 
@@ -282,9 +292,12 @@ class HeatUtility:
     ('low_pressure_steam', 1052.6315789473686, 0.02721274387089031, 0.006471190492497716)
            
     """
-    __slots__ = ('inlet_utility_stream', 'outlet_utility_stream', 'duty',
-                 'flow', 'cost', 'heat_transfer_efficiency', 
-                 'unit', 'hxn_ok', 'agent', 'unit_duty')
+    __slots__ = (
+        'inlet_utility_stream', 'outlet_utility_stream', 'agent', 
+        'duty', 'flow', 'cost', 'heat_transfer_efficiency', 
+        'unit', 'hxn_ok', 'unit_duty',
+        'oxygen_rich_inlet', # Only for fuels
+    )
     #: Minimum approach temperature difference. Used to assign
     #: the pinch temperature of the utility stream.
     dT: float = 5  
@@ -292,14 +305,23 @@ class HeatUtility:
     #: Units of measure for IPython display.
     display_units: DisplayUnits = DisplayUnits(duty='kJ/hr', flow='kmol/hr', cost='USD/hr')
 
-    #: Used broadly throughout BioSTEAM utilities.
+    #: Used broadly throughout utilities.
     thermo_water: Thermo = Thermo(['Water'])
     
-    #: Used for BioSTEAM refrigeration utility.
+    #: Used for refrigeration utility.
     thermo_propane: Thermo = Thermo(['Propane'])
 
+    #: Used for refrigeration utility.
+    thermo_propylene: Thermo = Thermo(['Propylene'])
+    
+    #: Used for refrigeration utility.
+    thermo_ethylene: Thermo = Thermo(['Ethylene'])
+
+    #: Used exclusively for furnaces.
+    thermo_natural_gas: Thermo = Thermo(['Methane', 'N2', 'CO2', 'O2', 'H2O'])
+
     #: Characterization factor data (value and units) by agent ID and impact key.
-    characterization_factors: dict[tuple[str, str], tuple[float, AbsoluteUnitsOfMeasure]] = {}
+    characterization_factors: dict[tuple[str, str], tuple[float, UnitsOfMeasure]] = {}
     
     #: All cooling utilities available.
     cooling_agents: list[UtilityAgent]
@@ -447,11 +469,11 @@ class HeatUtility:
         agent = self.agent
         CF, basis_units = self.get_CF(agent.ID, key)
         if basis_units == 'kg':
-            return self.flow * agent.MW, basis_units
+            return self.flow * agent.MW, basis_units + '/hr'
         elif basis_units == 'mol':
-            return self.flow, basis_units
+            return self.flow, basis_units + '/hr'
         elif basis_units == 'kJ':
-            return self.duty, basis_units
+            return self.duty, basis_units + '/hr'
         else:
             raise RuntimeError("unknown error")
 
@@ -535,9 +557,19 @@ class HeatUtility:
             regeneration_price = 0.3171,
             heat_transfer_efficiency = 0.85,
         ) 
+        natural_gas = UtilityAgent(
+            'natural_gas',
+            Methane=1, T=298.15, P=200 * 101325, phase='g', 
+            thermo=cls.thermo_natural_gas,
+            heat_transfer_efficiency = 0.90, # Heat loss to environment
+            regeneration_price=3.49672,
+            T_limit=405, # Must be reasonably higher than the emission's dew point at 500 psig (401 K)
+            isfuel=True,
+        ) 
         cls.heating_agents = [low_pressure_steam, 
                               medium_pressure_steam, 
-                              high_pressure_steam]
+                              high_pressure_steam,
+                              natural_gas]
         
     @classmethod
     def default_cooling_agents(cls):
@@ -549,13 +581,15 @@ class HeatUtility:
             thermo=thermo_water,
             T_limit = 324.817,
             regeneration_price = 4.8785e-4,
+            dT=2.,
         )
         chilled_water = UtilityAgent(
             'chilled_water',
             Water=1, T=280.372, P=101325,
             thermo=thermo_water,
             T_limit = 300.372,
-            heat_transfer_price = 5e-6
+            heat_transfer_price = 5e-6,
+            dT=2.,
         )
         chilled_brine = UtilityAgent(
             'chilled_brine',
@@ -563,20 +597,44 @@ class HeatUtility:
             thermo=thermo_water,
             T_limit = 275.372,
             heat_transfer_price = 8.145e-6,
+            dT=2.,
         )
         propane = UtilityAgent(
             'propane',
             Propane=1,
             thermo=cls.thermo_propane,
-            T=273.15 - 42.11,
-            P=101325,
+            T=238.70,
+            P=cls.thermo_propane.chemicals.Propane.Psat(238.70),
             heat_transfer_price = 13.17e-6,
             phase='l',
+            dT=1.,
+        )
+        propylene = UtilityAgent(
+            'propylene',
+            Propylene=1,
+            thermo=cls.thermo_propylene,
+            T=227.59,
+            P=cls.thermo_propylene.chemicals.Propylene.Psat(227.59),
+            heat_transfer_price = 16.54e-6, # Lever rule with -30 and -90 F prices
+            phase='l',
+            dT=1.,
+        )
+        ethylene = UtilityAgent(
+            'ethylene',
+            Ethylene=1,
+            thermo=cls.thermo_ethylene,
+            T=172.04,
+            P=cls.thermo_ethylene.chemicals.Ethylene.Psat(172.04),
+            heat_transfer_price = 33.2e-06,
+            phase='l',
+            dT=1.,
         )
         cls.cooling_agents = [cooling_water, 
                               chilled_water, 
                               chilled_brine,
-                              propane]
+                              propane,
+                              propylene,
+                              ethylene]
 
     def copy(self) -> HeatUtility:
         hu = HeatUtility()
@@ -601,6 +659,9 @@ class HeatUtility:
         self.duty *= factor
         self.cost *= factor
         self.inlet_utility_stream.mol *= factor
+        if self.agent.isfuel: 
+            self.outlet_utility_stream.mol *= factor
+            self.oxygen_rich_inlet.mol *= factor
         # No need to factor the outlet utility stream
         # because it shares the same flow rate data as the inlet
 
@@ -610,9 +671,14 @@ class HeatUtility:
             agent = self.agent
             dump = agent.utility_stream_dump
             if len(dump) < 1000:
-                dump.append(
-                    (self.inlet_utility_stream, self.outlet_utility_stream)
-                )
+                if agent.isfuel:
+                    dump.append(
+                        (self.inlet_utility_stream, self.outlet_utility_stream, self.oxygen_rich_inlet)
+                    )
+                else:
+                    dump.append(
+                        (self.inlet_utility_stream, self.outlet_utility_stream)
+                    )
         self.cost = self.flow = self.duty = self.unit_duty = 0
         self.outlet_utility_stream = self.inlet_utility_stream = self.agent = None
         
@@ -622,7 +688,7 @@ class HeatUtility:
             return
         self.load_agent(agent)
         heat_transfer_efficiency = self.heat_transfer_efficiency or agent.heat_transfer_efficiency
-        if agent.T_limit: raise ValueError('agent must work by latent heat to set by flow rate')
+        if agent.T_limit or agent.isfuel: raise ValueError('agent must work by latent heat to set by flow rate')
         if self.inlet_utility_stream.phase == 'l' :
             self.outlet_utility_stream.phase = 'g'
             dh = -agent._get_property('Hvap', nophase=True)
@@ -679,7 +745,32 @@ class HeatUtility:
         ## Calculate utility requirement ##
         heat_transfer_efficiency = self.heat_transfer_efficiency or agent.heat_transfer_efficiency
         duty = unit_duty / heat_transfer_efficiency
-        if agent.T_limit:
+        if agent.isfuel:
+            T_emissions = self.get_outlet_temperature(
+                T_pinch_out, agent.T_limit, iscooling
+            )
+            feed = self.inlet_utility_stream
+            emissions = self.outlet_utility_stream
+            emissions.copy_like(feed)
+            emissions.P = 101325
+            reactions = agent.chemicals.get_combustion_reactions()
+            reactions.force_reaction(emissions)
+            O2_consumption = -emissions.imol['O2']
+            oxygen_rich_gas = self.oxygen_rich_inlet
+            z_O2 = oxygen_rich_gas.imol['O2'] / oxygen_rich_gas.F_mol
+            oxygen_rich_gas.F_mol = O2_consumption / z_O2
+            emissions.mol += oxygen_rich_gas.mol
+            F_emissions = emissions.F_mass
+            z_CO2 = emissions.imass['CO2'] / F_emissions
+            z_CO2_target = 0.055 # Usually between 4 - 7 for biomass and natural gas (https://www.sciencedirect.com/science/article/pii/S0957582021005127)
+            F_emissions_new = z_CO2 * F_emissions / z_CO2_target
+            dF_emissions = F_emissions_new - F_emissions
+            oxygen_rich_gas.F_mass = F_mass_O2_new = oxygen_rich_gas.F_mass + dF_emissions
+            emissions.mol += oxygen_rich_gas.mol * (dF_emissions / F_mass_O2_new)
+            emissions.T = T_emissions
+            emissions.P = 3548325.0 # 500 psig
+            dh = feed.Hnet - emissions.Hnet
+        elif agent.T_limit:
             # Temperature change
             self.outlet_utility_stream.T = T_outlet = self.get_outlet_temperature(
                 T_pinch_out, agent.T_limit, iscooling
@@ -695,7 +786,11 @@ class HeatUtility:
                 self.outlet_utility_stream.phase = 'l'
         
         # Update utility flow
-        self.outlet_utility_stream.mol[:] = F_mol = duty / dh
+        F_mol = duty / dh
+        self.inlet_utility_stream.mol[:] *= F_mol
+        if agent.isfuel: 
+            emissions.mol[:] *= F_mol
+            oxygen_rich_gas.mol[:] *= F_mol
         
         # Update results
         self.unit_duty = unit_duty
@@ -782,7 +877,7 @@ class HeatUtility:
         
         """
         for agent in cls.heating_agents:
-            if T_pinch < agent.T: return agent
+            if T_pinch < agent.T - agent.dT or agent.isfuel: return agent
         raise RuntimeError(f'no heating agent that can heat over {T_pinch} K')    
 
     @classmethod
@@ -796,23 +891,33 @@ class HeatUtility:
         
         """
         for agent in cls.cooling_agents:
-            if T_pinch > agent.T: return agent
+            if T_pinch > agent.T + agent.dT: return agent
         raise RuntimeError(f'no cooling agent that can cool under {T_pinch} K')    
 
     def load_agent(self, agent: UtilityAgent):
         """Initialize utility streams with given agent."""
         if self.agent is agent: 
+            self.inlet_utility_stream.copy_like(agent)
+            if not agent.T_limit: self.outlet_utility_stream.copy_thermal_condition(agent)
             return
         elif self.agent: 
             self.empty()
         if agent.utility_stream_dump:
-            self.inlet_utility_stream, self.outlet_utility_stream = agent.utility_stream_dump.pop()
+            if agent.isfuel:
+                self.inlet_utility_stream, self.outlet_utility_stream, self.oxygen_rich_inlet = agent.utility_stream_dump.pop()
+                self.oxygen_rich_inlet.reset_flow(O2=21, N2=79, phase='g', units='kg/hr')
+            else:
+                self.inlet_utility_stream, self.outlet_utility_stream = agent.utility_stream_dump.pop()
             # Prevent errors where utility streams are altered
             self.inlet_utility_stream.copy_like(agent) 
-            self.outlet_utility_stream.copy_thermal_condition(agent)
+            if not agent.T_limit: self.outlet_utility_stream.copy_thermal_condition(agent)
         else:
             self.inlet_utility_stream = agent.to_stream()
-            self.outlet_utility_stream = self.inlet_utility_stream.flow_proxy()
+            if agent.isfuel:
+                self.outlet_utility_stream = agent.to_stream()
+                self.oxygen_rich_inlet = Stream(O2=21, N2=79, phase='g', units='kg/hr', thermo=agent.thermo)
+            else:
+                self.outlet_utility_stream = self.inlet_utility_stream.flow_proxy()
         self.agent = agent
 
     def mix_from(self, heat_utilities: Iterable[HeatUtility]):
