@@ -43,6 +43,7 @@ __all__ = (
     'heat_exchanger_operation',
     'connect_by_ID',
     'default_utilities',
+    'default_CEPCI',
     'default',
 )
 
@@ -131,7 +132,7 @@ def rename_unit(unit, area):
     * U = Other units
     * V = Valve
     * J = Junction, not a physical unit (serves to adjust streams)
-    * PS = Process specificiation, not a physical unit (serves to adjust streams)
+    * PS = Process specification, not a physical unit (serves to adjust streams)
     
     Examples
     --------
@@ -181,7 +182,7 @@ def rename_units(units, area):
     * T = Tank or bin for storage
     * U = Other units
     * J = Junction, not a physical unit (serves to adjust streams)
-    * PS = Process specificiation, not a physical unit (serves to adjust streams)
+    * PS = Process specification, not a physical unit (serves to adjust streams)
     
     Examples
     --------
@@ -483,7 +484,10 @@ def default_utilities():
     """Reset utilities back to BioSTEAM's defaults."""
     bst.HeatUtility.default_agents()
     bst.PowerUtility.default_price()
-    
+
+def default_CEPCI():
+    bst.CE = 567.5
+
 def default(utilities=True, CEPCI=True, flowsheet=False):
     """
     Reset utilities, flowsheets, and chemical plant cost index (CEPCI) back to 
@@ -491,7 +495,7 @@ def default(utilities=True, CEPCI=True, flowsheet=False):
     
     """
     if utilities: default_utilities()
-    if CEPCI: bst.CE = 567.5
+    if CEPCI: default_CEPCI()
     for i in (bst.Stream, bst.Unit, bst.System): i.ticket_numbers.clear()
     if flowsheet: 
         for i in bst.main_flowsheet.flowsheet: i.clear(False)
