@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -12,8 +12,7 @@ from .splitting import Splitter
 
 __all__ = ('VibratingScreen',)
 
-@cost('Area', units='ft^2', ub=200, CE=567, cost=1010, n=0.91, BM=1.73, N='Number of screens',
-      fsize=lambda self, _: self.ins[0].F_mass/(self.capacity*self.mesh_opening))
+@cost('Area', ub=200, CE=567, cost=1010, n=0.91, BM=1.73, N='Number of screens')
 class VibratingScreen(Splitter):
     # Assume 3-deck vibrating screen
     
@@ -21,4 +20,9 @@ class VibratingScreen(Splitter):
     capacity = 0.2*907.18474
     
     #: Mesh opening (mm)
-    mesh_opening = 8
\ No newline at end of file
+    mesh_opening = 8
+    
+    _units = {'Area': 'ft^2'}
+    
+    def _design(self):
+        self.design_results['Area'] = self.ins[0].F_mass/(self.capacity*self.mesh_opening)
\ No newline at end of file
