@@ -25,14 +25,17 @@ References
 
 """
 import biosteam as bst
+import numpy as np
 from warnings import warn
 from math import log, exp, ceil
 from typing import NamedTuple, Tuple, Callable, Dict
+from thermosteam.constants import R
 from .heat_exchange import HX
 from ..utils import list_available_names
 from ..exceptions import DesignWarning, bounds_warning
 from .. import Unit
-from .._graphics import compressor_graphics
+from thermosteam._graphics import compressor_graphics
+from thermosteam import VariableNode
 
 __all__ = (
     'Compressor',
@@ -126,10 +129,10 @@ class Compressor(Unit, isabstract=True):
             ),
     }
 
-    def __init__(self, ID='', ins=None, outs=(), thermo=None, *, 
-                 P, eta=0.7, vle=False, compressor_type=None, 
-                 driver=None, material=None, driver_efficiency=None):
-        Unit.__init__(self, ID, ins, outs, thermo)
+    def _init(self, 
+            P, eta=0.7, vle=False, compressor_type=None, 
+            driver=None, material=None, driver_efficiency=None
+        ):
         self.P = P  #: Outlet pressure [Pa].
         self.eta = eta  #: Isentropic efficiency.
 
@@ -542,6 +545,57 @@ class IsentropicCompressor(Compressor, new_graphics=False):
     Utility cost                                  USD/hr       9.66e-05
 
     """
+    equation_node_names = (
+        'overall_material_balance_node',
+        'energy_balance_node',
+        'isentropic_compression_phenomenode',
+    )
+    _energy_variable = 'T'
+
+    @property
+    def T_node(self):
+        if hasattr(self, '_T_node'): return self._T_node
+        self._T_node = var = VariableNode(f"{self.node_tag}.T", lambda: self.outs[0].T)
+        return var 
+    
+    def get_E_node(self, stream):
+        return self.T_node
+    
+    @property
+    def E_node(self):
+        return self.T_node
+
+    @property
+    def Q_node(self):
+        if hasattr(self, '_Q_node'): return self._Q_node
+        self._Q_node = var = VariableNode(f"{self.node_tag}.Q", lambda: self.Q)
+        return var
+
+    def initialize_isentropic_compression_phenomenode(self):
+        self.isentropic_compression_phenomenode.set_equations(
+            inputs=(
+                self.T_node, 
+                *[i.T_node for i in (*self.ins, *self.outs)],
+                *[i.F_node for i in (*self.ins, *self.outs)],
+            ),
+            outputs=[self.Q_node],
+        )
+
+    def initialize_overall_material_balance_node(self):
+        self.overall_material_balance_node.set_equations(
+            inputs=[j for i in self.ins if (j:=i.F_node)],
+            outputs=[i.F_node for i in self.outs],
+        )
+        
+    def initialize_energy_balance_node(self):
+        self.energy_balance_node.set_equations(
+            inputs=(
+                self.Q_node,
+                *[i.T_node for i in (*self.ins, *self.outs)],
+                *[i.F_node for i in (*self.ins, *self.outs)],
+            ),
+            outputs=[self.E_node],
+        )
 
     def _run(self):
         feed = self.ins[0]
@@ -549,23 +603,71 @@ class IsentropicCompressor(Compressor, new_graphics=False):
         out.copy_like(feed)
         out.P = self.P
         out.S = feed.S
-        if self.vle is True:
-            out.vle(S=out.S, P=out.P)
-            T_isentropic = out.T
-        else:
-            T_isentropic = out.T
-        self.T_isentropic = T_isentropic
+        self.T_isentropic = out.T
+        if self.vle is True: out.vle(S=out.S, P=out.P)
         dH_isentropic = out.H - feed.H
         self.design_results['Ideal power'] = dH_isentropic / 3600. # kW
         self.design_results['Ideal duty'] = 0.
-        dH_actual = dH_isentropic / self.eta
+        self.Q = dH_actual = dH_isentropic / self.eta
         out.H = feed.H + dH_actual        
         if self.vle is True: out.vle(H=out.H, P=out.P)
         
+    def _mass_and_energy_balance_specifications(self):
+        return 'Compressor', [
+            ('Isentropic efficiency', 100 * self.eta, '%'),
+            ('P', self.P, 'Pa'),
+        ]
+        
     def _design(self):
         super()._design()
         self._set_power(self.design_results['Ideal power'] / self.eta)
+    
+    def _get_energy_departure_coefficient(self, stream):
+        feed = self.ins[0]
+        source = feed.source
+        if source is None or source._energy_variable != 'T': return None
+        return (stream, -stream.C)
+    
+    def _update_nonlinearities(self):
+        feed = self.ins[0]
+        out = self.outs[0]
+        if len(out.phases) > 1:
+            raise NotImplementedError('energy departure equation with multiple phase not yet implemented for isentropic compressors')
+        mixture = self.mixture
+        T_isentropic = getattr(self, 'T_isentropic', feed.T)
+        self.T_isentropic = mixture.solve_T_at_SP(out.phase, out.mol, feed.S, T_isentropic, out.P)
+        dH_isentropic = mixture('H', out, T=self.T_isentropic) - feed.H
+        self.design_results['Ideal power'] = dH_isentropic / 3600. # kW
+        self.design_results['Ideal duty'] = 0.
+        self.Q = dH_isentropic / self.eta
         
+    def _create_energy_departure_equations(self):
+        feed = self.ins[0]
+        out = self.outs[0]
+        coeff = {self: out.C}
+        feed._update_energy_departure_coefficient(coeff)
+        return [(coeff, feed.H - out.H + self.Q)]
+    
+    def _create_material_balance_equations(self, composition_sensitive):
+        fresh_inlets, process_inlets, equations = self._begin_equations(composition_sensitive)
+        outlet, = self.outs
+        if len(outlet.phases) > 1:
+            raise NotImplementedError('energy departure equation with multiple phase not yet implemented for isentropic compressors')
+        ones = np.ones(self.chemicals.size)
+        minus_ones = -ones
+        zeros = np.zeros(self.chemicals.size)
+        
+        # Overall flows
+        eq_overall = {outlet: ones}
+        for i in process_inlets: eq_overall[i] = minus_ones
+        equations.append(
+            (eq_overall, sum([i.mol for i in fresh_inlets], zeros))
+        )
+        return equations
+    
+    def _update_energy_variable(self, departure):
+        self.outs[0].T += departure
+
 
 class PolytropicCompressor(Compressor, new_graphics=False):
     """
@@ -590,7 +692,7 @@ class PolytropicCompressor(Compressor, new_graphics=False):
         for real gases at high pressure ratios.
     n_steps: int
         Number of virtual steps used in numerical integration for hundseid method.
-    type: str
+    compressor_type: str
         Type of compressor : blower/centrifugal/reciprocating. If None, the type
         will be determined automatically.
 
@@ -704,9 +806,9 @@ class PolytropicCompressor(Compressor, new_graphics=False):
 
     """
     available_methods = {'schultz', 'hundseid'}
-    def __init__(self, ID='', ins=None, outs=(), thermo=None, *, P, eta=0.7, 
+    def _init(self, P, eta=0.7, 
                  vle=False, compressor_type=None, method=None, n_steps=None):
-        Compressor.__init__(self, ID=ID, ins=ins, outs=outs, thermo=thermo, P=P, eta=eta, vle=vle, compressor_type=compressor_type)
+        Compressor._init(self, P=P, eta=eta, vle=vle, compressor_type=compressor_type)
         self.method = "schultz" if method is None else method
         self.n_steps = 100 if n_steps is None else n_steps
         
@@ -851,29 +953,27 @@ class MultistageCompressor(Unit):
         flow (kmol/hr): H2  1
     
     >>> K.results()
-    Multistage compressor                           Units                      K
-    Electricity         Power                          kW                      0
-                        Cost                       USD/hr                      0
-    High pressure steam Duty                        kJ/hr                   5.68
-                        Flow                      kmol/hr               0.000177
-                        Cost                       USD/hr                5.6e-05
-    Chilled water       Duty                        kJ/hr              -1.12e+04
-                        Flow                      kmol/hr                   7.42
-                        Cost                       USD/hr                 0.0559
-    Design              Type                               Multistage compressor
-                        Area                         ft^2                   1.54
-                        Tube side pressure drop       psi                     12
-                        Shell side pressure drop      psi                     20
-    Purchase cost       K k1 - Compressor(s)          USD               1.45e+04
-                        K h1 - Double pipe            USD                    568
-                        K k2 - Compressor(s)          USD               1.46e+04
-                        K h2 - Double pipe            USD                    675
-                        K k3 - Compressor(s)          USD               1.48e+04
-                        K h3 - Double pipe            USD                    956
-                        K k4 - Compressor(s)          USD               1.52e+04
-                        K h4 - Double pipe            USD               1.78e+03
-    Total purchase cost                               USD                6.3e+04
-    Utility cost                                   USD/hr                  0.056
+    Multistage compressor                       Units                      K
+    Electricity         Power                      kW                      0
+                        Cost                   USD/hr                      0
+    High pressure steam Duty                    kJ/hr                   5.68
+                        Flow                  kmol/hr               0.000177
+                        Cost                   USD/hr                5.6e-05
+    Chilled water       Duty                    kJ/hr              -1.12e+04
+                        Flow                  kmol/hr                   7.42
+                        Cost                   USD/hr                 0.0559
+    Design              Type                           Multistage compressor
+                        Area                     ft^2                   1.54
+    Purchase cost       K k1 - Compressor(s)      USD               1.45e+04
+                        K h1 - Double pipe        USD                    568
+                        K k2 - Compressor(s)      USD               1.46e+04
+                        K h2 - Double pipe        USD                    675
+                        K k3 - Compressor(s)      USD               1.48e+04
+                        K h3 - Double pipe        USD                    956
+                        K k4 - Compressor(s)      USD               1.52e+04
+                        K h4 - Double pipe        USD               1.78e+03
+    Total purchase cost                           USD                6.3e+04
+    Utility cost                               USD/hr                  0.056
 
     Show the fluid state at the outlet of each heat exchanger:
     
@@ -917,34 +1017,32 @@ class MultistageCompressor(Unit):
         flow (kmol/hr): H2  1
     
     >>> K.results()
-    Multistage compressor                           Units                     K2
-    Electricity         Power                          kW                      0
-                        Cost                       USD/hr                      0
-    High pressure steam Duty                        kJ/hr                   6.48
-                        Flow                      kmol/hr               0.000202
-                        Cost                       USD/hr               6.39e-05
-    Chilled water       Duty                        kJ/hr              -5.63e+03
-                        Flow                      kmol/hr                   3.73
-                        Cost                       USD/hr                 0.0282
-    Cooling water       Duty                        kJ/hr              -7.15e+03
-                        Flow                      kmol/hr                   4.88
-                        Cost                       USD/hr                0.00238
-    Design              Type                               Multistage compressor
-                        Area                         ft^2                   1.14
-                        Tube side pressure drop       psi                     15
-                        Shell side pressure drop      psi                     25
-    Purchase cost       K2 k1 - Compressor(s)         USD               1.11e+04
-                        K2 h1 - Double pipe           USD                    297
-                        K2 k2 - Compressor(s)         USD                1.3e+04
-                        K2 h2 - Double pipe           USD                    199
-                        K2 k3 - Compressor(s)         USD               1.44e+04
-                        K2 h3 - Double pipe           USD                    129
-                        K2 k4 - Compressor(s)         USD               1.64e+04
-                        K2 h4 - Double pipe           USD                    753
-                        K2 k5 - Compressor(s)         USD               1.45e+04
-                        K2 h5 - Double pipe           USD               2.03e+03
-    Total purchase cost                               USD               7.27e+04
-    Utility cost                                   USD/hr                 0.0306
+    Multistage compressor                        Units                     K2
+    Electricity         Power                       kW                      0
+                        Cost                    USD/hr                      0
+    High pressure steam Duty                     kJ/hr                   6.48
+                        Flow                   kmol/hr               0.000202
+                        Cost                    USD/hr               6.39e-05
+    Chilled water       Duty                     kJ/hr              -5.63e+03
+                        Flow                   kmol/hr                   3.73
+                        Cost                    USD/hr                 0.0282
+    Cooling water       Duty                     kJ/hr              -7.15e+03
+                        Flow                   kmol/hr                   4.88
+                        Cost                    USD/hr                0.00238
+    Design              Type                            Multistage compressor
+                        Area                      ft^2                   1.14
+    Purchase cost       K2 k1 - Compressor(s)      USD               1.11e+04
+                        K2 h1 - Double pipe        USD                    297
+                        K2 k2 - Compressor(s)      USD                1.3e+04
+                        K2 h2 - Double pipe        USD                    199
+                        K2 k3 - Compressor(s)      USD               1.44e+04
+                        K2 h3 - Double pipe        USD                    129
+                        K2 k4 - Compressor(s)      USD               1.64e+04
+                        K2 h4 - Double pipe        USD                    753
+                        K2 k5 - Compressor(s)      USD               1.45e+04
+                        K2 h5 - Double pipe        USD               2.03e+03
+    Total purchase cost                            USD               7.27e+04
+    Utility cost                                USD/hr                 0.0306
 
     Show the fluid state at the outlet of each heat exchanger:
     
@@ -1011,34 +1109,32 @@ class MultistageCompressor(Unit):
         flow (kmol/hr): H2  1
     
     >>> K.results()
-    Multistage compressor                           Units                     K2
-    Electricity         Power                          kW                      0
-                        Cost                       USD/hr                      0
-    High pressure steam Duty                        kJ/hr                   6.48
-                        Flow                      kmol/hr               0.000202
-                        Cost                       USD/hr               6.39e-05
-    Chilled water       Duty                        kJ/hr              -5.63e+03
-                        Flow                      kmol/hr                   3.73
-                        Cost                       USD/hr                 0.0282
-    Cooling water       Duty                        kJ/hr              -7.15e+03
-                        Flow                      kmol/hr                   4.88
-                        Cost                       USD/hr                0.00238
-    Design              Type                               Multistage compressor
-                        Area                         ft^2                   1.14
-                        Tube side pressure drop       psi                     15
-                        Shell side pressure drop      psi                     25
-    Purchase cost       K2 k1 - Compressor(s)         USD               1.11e+04
-                        K2 h1 - Double pipe           USD                    297
-                        K2 k2 - Compressor(s)         USD                1.3e+04
-                        K2 h2 - Double pipe           USD                    199
-                        K2 k3 - Compressor(s)         USD               1.44e+04
-                        K2 h3 - Double pipe           USD                    129
-                        K2 k4 - Compressor(s)         USD               1.64e+04
-                        K2 h4 - Double pipe           USD                    753
-                        K2 k5 - Compressor(s)         USD               1.45e+04
-                        K2 h5 - Double pipe           USD               2.03e+03
-    Total purchase cost                               USD               7.27e+04
-    Utility cost                                   USD/hr                 0.0306
+    Multistage compressor                        Units                     K2
+    Electricity         Power                       kW                      0
+                        Cost                    USD/hr                      0
+    High pressure steam Duty                     kJ/hr                   6.48
+                        Flow                   kmol/hr               0.000202
+                        Cost                    USD/hr               6.39e-05
+    Chilled water       Duty                     kJ/hr              -5.63e+03
+                        Flow                   kmol/hr                   3.73
+                        Cost                    USD/hr                 0.0282
+    Cooling water       Duty                     kJ/hr              -7.15e+03
+                        Flow                   kmol/hr                   4.88
+                        Cost                    USD/hr                0.00238
+    Design              Type                            Multistage compressor
+                        Area                      ft^2                   1.14
+    Purchase cost       K2 k1 - Compressor(s)      USD               1.11e+04
+                        K2 h1 - Double pipe        USD                    297
+                        K2 k2 - Compressor(s)      USD                1.3e+04
+                        K2 h2 - Double pipe        USD                    199
+                        K2 k3 - Compressor(s)      USD               1.44e+04
+                        K2 h3 - Double pipe        USD                    129
+                        K2 k4 - Compressor(s)      USD               1.64e+04
+                        K2 h4 - Double pipe        USD                    753
+                        K2 k5 - Compressor(s)      USD               1.45e+04
+                        K2 h5 - Double pipe        USD               2.03e+03
+    Total purchase cost                            USD               7.27e+04
+    Utility cost                                USD/hr                 0.0306
 
     """
 
@@ -1049,13 +1145,10 @@ class MultistageCompressor(Unit):
         **HX._units,
     }
 
-    def __init__(
-            self, ID='', ins=None, outs=(), thermo=None, *,
-            pr=None, n_stages=None, eta=0.7, vle=False, compressor_type=None,
+    def _init(
+            self, pr=None, n_stages=None, eta=0.7, vle=False, compressor_type=None,
             compressors=None, hxs=None,
-    ):
-        super().__init__(ID=ID, ins=ins, outs=outs, thermo=thermo)
-
+        ):
         # setup option 1: list of compressors and list of heat exchangers
         if compressors is not None and hxs is not None:
             if not isinstance(compressors[0], Compressor):
