@@ -22,7 +22,7 @@ References
 """
 import numpy as np
 from . import utils
-from numba import njit
+from numba import njit, objmode
 import biosteam as bst
 from warnings import warn
 
@@ -46,6 +46,12 @@ def minimum_thickness_from_diameter(D):
     return 0.03125 * D + 0.125
 
 @njit(cache=True)
+def _compute_purchase_cost_of_trays(N_T, Di, CE):
+    F_CE = CE/500.
+    C_BT = compute_tray_base_purchase_cost(Di)
+    F_NT = compute_n_trays_factor(N_T)
+    return N_T * F_CE * F_NT * C_BT
+
 def compute_purchase_cost_of_trays(N_T, Di):
     """
     Return total cost of all trays at BioSTEAM's CEPCI.
@@ -64,12 +70,13 @@ def compute_purchase_cost_of_trays(N_T, Di):
     Plant Cost Index, `biosteam.CE`.
     
     """
-    F_CE = bst.CE/500.
-    C_BT = compute_tray_base_purchase_cost(Di)
-    F_NT = compute_n_trays_factor(N_T)
-    return N_T * F_CE * F_NT * C_BT
+    return _compute_purchase_cost_of_trays(N_T, Di, bst.CE)
 
 @njit(cache=True)
+def _compute_empty_tower_cost(W, CE):
+    logW = np.log(W)
+    return CE/500. * np.exp(7.2756 + 0.18255*logW + 0.02297*logW*logW)
+
 def compute_empty_tower_cost(W):
     """
     Return the cost [C_V; in USD] of an empty tower vessel at BioSTEAM's CEPCI.
@@ -85,10 +92,12 @@ def compute_empty_tower_cost(W):
     The purchase cost is given by [1]_. See source code for details.
     
     """
-    logW = np.log(W)
-    return bst.CE/500. * np.exp(7.2756 + 0.18255*logW + 0.02297*logW*logW)
+    return _compute_empty_tower_cost(W, bst.CE)
 
 @njit(cache=True)
+def _compute_plaform_ladder_cost(Di, L, CE):
+    return CE/500. * 300.9*Di**0.63316*L**0.80161
+
 def compute_plaform_ladder_cost(Di, L):
     """
     Return the cost [C_PL; in USD] of platforms and ladders at BioSTEAM's CEPCI.
@@ -105,7 +114,7 @@ def compute_plaform_ladder_cost(Di, L):
     The purchase cost is given by [1]_. See source code for details.
     
     """
-    return bst.CE/500. * 300.9*Di**0.63316*L**0.80161
+    return _compute_plaform_ladder_cost(Di, L, bst.CE)
 
 @njit(cache=True)
 def compute_tower_weight(Di, L, tv, rho_M):
@@ -336,7 +345,7 @@ def compute_max_vapor_velocity(C_sbf, sigma, rho_L, rho_V, F_F, A_ha):
     F_F : 
         Foaming factor
     A_ha : 
-        Ratio of open area, A_h, to active area, A_a
+        Ratio of open area, A_h, to active area, A_a.
     
     Notes
     -----
@@ -398,14 +407,14 @@ def compute_tower_diameter(V_vol, U_f, f, A_dn):
     The tower diameter is given by [3]_. See source code for details.
     
     """
-    Di = np.sqrt(4.*V_vol/(f*U_f*np.pi*(1.-A_dn)))
+    Di = np.sqrt(4. * V_vol/(f * U_f * np.pi* (1. - A_dn)))
     if Di < 0.914:
         # Make sure diameter is not too small
         Di = 0.914
     return Di
 
 @njit(cache=True)
-def compute_tower_height(TS, N_stages: int, top=True, bot=True):
+def compute_tower_height(TS, N_stages: int, top=True, bottom=True):
     """
     Return the height of a tower [H; in meter].
     
@@ -425,6 +434,6 @@ def compute_tower_height(TS, N_stages: int, top=True, bot=True):
     H = TS*N_stages/1000.
     if top:
         H += 1.2672
-    if bot:
+    if bottom:
         H += 3.
     return H 
\ No newline at end of file
