@@ -14,9 +14,11 @@
 
 """
 from .._unit import Unit
-from .._graphics import mixer_graphics
+from thermosteam._graphics import mixer_graphics
 import flexsolve as flx
 import biosteam as bst
+import numpy as np
+from typing import Optional
 
 __all__ = ('Mixer', 'SteamMixer', 'FakeMixer', 'MockMixer')
 
@@ -30,6 +32,8 @@ class Mixer(Unit):
         Inlet fluids to be mixed.
     outs : 
         Mixed outlet fluid.
+    rigorous :
+        Whether to perform vapor-liquid equilibrium.
     
     Notes
     -----
@@ -71,17 +75,152 @@ class Mixer(Unit):
     _N_ins = 2
     _ins_size_is_fixed = False
     
+    equation_node_names = (
+        'overall_material_balance_node', 
+        'energy_balance_node',
+    )
+    
+    @property
+    def _energy_variable(self):
+        phases = self.outs[0].phases
+        if phases == ('g', 'l'):
+            return 'B'
+        else:
+            return 'T'
+    
+    def initialize_overall_material_balance_node(self):
+        self.overall_material_balance_node.set_equations(
+            outputs=[i.F_node for i in self.outs],
+            inputs=[j for i in self.ins if (j:=i.F_node)],
+        )
+        
+    def initialize_energy_balance_node(self):
+        self.energy_balance_node.set_equations(
+            inputs=(
+                self.T_node, 
+                *[i.F_node for i in (*self.ins, *self.outs)],
+                *[j for i in self.ins if (j:=i.E_node)],
+            ),
+            outputs=[
+                j for i in self.outs if (j:=i.E_node)
+            ],
+        )
+    
+    @property
+    def T_node(self):
+        if hasattr(self, '_T_node'): return self._T_node
+        self._T_node = var = bst.VariableNode(f"{self.node_tag}.T", lambda: self.outs[0].T)
+        return var 
+        
+    @property
+    def E_node(self):
+        return self.T_node    
+    
     def _assert_compatible_property_package(self): 
         pass # Not necessary for mixing streams
     
-    def __init__(self, ID='', ins=None, outs=(), thermo=None, rigorous=False):
-        Unit.__init__(self, ID, ins, outs, thermo)
+    def _init(self, rigorous: Optional[bool]=False,
+              conserve_phases: Optional[bool]=False):
         self.rigorous = rigorous
+        self.conserve_phases = conserve_phases
     
     def _run(self):
         s_out, = self.outs
-        s_out.mix_from(self.ins, vle=self.rigorous)
-        
+        s_out.mix_from(self.ins, vle=self.rigorous,
+                       conserve_phases=getattr(self, 'conserve_phases', None))
+        V = s_out.vapor_fraction
+        if V == 0:
+            self.B = 0
+        elif V == 1:
+            self.B = np.inf
+        else:
+            self.B = V / (1 - V)
+    
+    def _get_energy_departure_coefficient(self, stream):
+        if stream.phases == ('g', 'l'):
+            vapor, liquid = stream
+            if vapor.isempty():
+                with liquid.temporary_phase('g'): coeff = liquid.H
+            else:
+                coeff = -vapor.h * liquid.F_mol
+        else:
+            coeff = -stream.C
+        return (self, coeff)
+    
+    def _create_energy_departure_equations(self):
+        # Ll: C1dT1 - Ce2*dT2 - Cr0*dT0 - hv2*L2*dB2 = Q1 - H_out + H_in
+        # gl: hV1*L1*dB1 - hv2*L2*dB2 - Ce2*dT2 - Cr0*dT0 = Q1 + H_in - H_out
+        outlet = self.outs[0]
+        phases = outlet.phases
+        if phases == ('g', 'l'):
+            vapor, liquid = outlet
+            coeff = {}
+            if vapor.isempty():
+                with liquid.temporary_phase('g'): coeff[self] = liquid.H
+            else:
+                coeff[self] = vapor.h * liquid.F_mol
+        else:
+            coeff = {self: outlet.C}
+        for i in self.ins: i._update_energy_departure_coefficient(coeff)
+        return [(coeff, self.H_in - self.H_out)]
+    
+    def _create_material_balance_equations(self, composition_sensitive):
+        fresh_inlets, process_inlets, equations = self._begin_equations(composition_sensitive)
+        outlet, = self.outs
+        if len(outlet) == 1:
+            ones = np.ones(self.chemicals.size)
+            minus_ones = -ones
+            zeros = np.zeros(self.chemicals.size)
+            
+            # Overall flows
+            eq_overall = {outlet: ones}
+            for i in process_inlets: eq_overall[i] = minus_ones
+            equations.append(
+                (eq_overall, sum([i.mol for i in fresh_inlets], zeros))
+            )
+        else:
+            top, bottom = outlet
+            ones = np.ones(self.chemicals.size)
+            minus_ones = -ones
+            zeros = np.zeros(self.chemicals.size)
+            
+            # Overall flows
+            eq_overall = {}
+            for i in outlet: 
+                eq_overall[i] = ones
+            for i in process_inlets:
+                eq_overall[i] = minus_ones
+            equations.append(
+                (eq_overall, sum([i.mol for i in fresh_inlets], zeros))
+            )
+            
+            # Top to bottom flows
+            B = self.B
+            eq_outs = {}
+            if B == np.inf:
+                eq_outs[bottom] = ones
+            elif B == 0:
+                eq_outs[top] = ones
+            else:
+                bp = outlet.bubble_point_at_P()
+                outlet.T = bp.T
+                S = bp.K * B
+                eq_outs[top] = ones
+                eq_outs[bottom] = -S
+            equations.append(
+                (eq_outs, zeros)
+            )
+        return equations
+    
+    def _update_energy_variable(self, departure):
+        phases = self.outs[0].phases
+        if phases == ('g', 'l'):
+            self.B += departure
+        else:
+            self.outs[0].T += departure
+
+    def _update_nonlinearities(self): pass
+
 
 class SteamMixer(Unit):
     """
@@ -167,11 +306,9 @@ class SteamMixer(Unit):
     _ins_size_is_fixed = False
     _graphics = mixer_graphics
     installation_cost = purchase_cost = 0.
-    def __init__(self, ID='', ins=None, outs=(), thermo=None, *, 
-                 P, T=None, solids_loading=None, 
-                 liquid_IDs=['7732-18-5'], solid_IDs=None,
-                 solids_loading_includes_steam=None):
-        Unit.__init__(self, ID, ins, outs, thermo)
+    def _init(self, P, T=None, solids_loading=None, 
+             liquid_IDs=['7732-18-5'], solid_IDs=None,
+             solids_loading_includes_steam=None):
         self.P = P
         self.T = T
         self.solids_loading = solids_loading
