@@ -106,11 +106,9 @@ class InternalCirculationRx(bst.MixTank):
     auxiliary_unit_names = ('heat_exchanger', 'effluent_pump', 'sludge_pump')
 
 
-    def __init__(self, ID='', ins=None, outs=(), thermo=None, *,
-                 method='lumped', OLRall=1.25, Y_biogas=0.86, Y_biomass=0.05, biodegradability={},
-                 vessel_type='IC', vessel_material='Stainless steel',
-                 V_wf=0.8, kW_per_m3=0., T=35+273.15, hxn_ok=False, **kwargs):
-        bst.Unit.__init__(self, ID, ins, outs, thermo)
+    def _init(self, method='lumped', OLRall=1.25, Y_biogas=0.86, Y_biomass=0.05, biodegradability={},
+              vessel_type='IC', vessel_material='Stainless steel',
+              V_wf=0.8, kW_per_m3=0., T=35+273.15, hxn_ok=False, **kwargs):
         self.method = method
         self.OLRall = OLRall
         self.Y_biogas = Y_biogas
@@ -137,7 +135,6 @@ class InternalCirculationRx(bst.MixTank):
         self.hxn_ok = hxn_ok
         for k, v in kwargs.items(): setattr(self, k, v)
 
-
     def _refresh_rxns(self, Y_biogas=None, Y_biomass=None):
         Y_biogas = Y_biogas if Y_biogas else self.Y_biogas
         Y_biomass = Y_biomass if Y_biomass else self.Y_biomass
@@ -307,6 +304,7 @@ class InternalCirculationRx(bst.MixTank):
         hx.ins[0].P = hx.outs[0].P = ins0.P
         hx.simulate_as_auxiliary_exchanger(ins=hx.ins, outs=hx.outs, 
                                            scale=1. / self.parallel.get('self', 1.),
+                                           vle=False,
                                            hxn_ok=self.hxn_ok)
         
         for p in (self.effluent_pump, self.sludge_pump): p.simulate()
