@@ -10,7 +10,7 @@
 import pytest
 import numpy as np
 import biosteam as bst
-from biosteam._network import Network
+from thermosteam.network import Network
 from numpy.testing import assert_allclose
 from biosteam import (
     main_flowsheet as f,
@@ -85,6 +85,27 @@ def test_simple_recycle_loop():
     assert_allclose(x_nested_solution, x_flat_solution, rtol=2e-2)
     f.clear()
 
+def test_single_unit_recycle_loop():
+    f.set_flowsheet('single_unit_recycle_loop')
+    settings.set_thermo(['Water', 'Ethanol'], cache=True)
+    feed = Stream('feed', Water=1000, Ethanol=500)
+    recycle = Stream('liquid_recycle')
+    liquid_product = Stream('liquid_product')
+    vapor_product = Stream('vapor_product')
+    stage = bst.StageEquilibrium(
+        'stage', ins=[feed, recycle], outs=[vapor_product, recycle, liquid_product], 
+        B=1, bottom_split=0.4, phases=('g', 'l')
+    )
+    single_unit_recycle_loop_sys = f.create_system('single_unit_recycle_loop_sys')
+    network = single_unit_recycle_loop_sys._to_network()
+    actual_network = Network(
+        [stage], recycle=recycle
+    )
+    assert network == actual_network
+    solution = [583.103718, 59.391173]
+    single_unit_recycle_loop_sys.simulate()
+    assert_allclose(solution, recycle.mol, rtol=2e-2)
+
 def test_unconnected_case():
     f.set_flowsheet('unconnected_case')
     settings.set_thermo(['Water'], cache=True)
@@ -374,12 +395,10 @@ def test_two_recycle_loops_with_complete_overlap():
          P3,
          Network(
             [M1,
-             Network(
-                 [M2,
-                  S2],
-                 recycle=inner_recycle),
+             M2,
+             S2,
              S1],
-            recycle=recycle)])
+            recycle={recycle, inner_recycle})])
     assert network == actual_network
     recycle_loop_sys.simulate()
     x_nested_solution = np.vstack([recycle.mol, inner_recycle.mol])
@@ -417,31 +436,27 @@ def test_two_recycle_loops_with_partial_overlap():
          P3,
          Network(
             [M1,
-             Network(
-                 [M2,
-                  S2,
-                  S3],
-                 recycle=inner_recycle),
-             S1],
-            recycle=recycle)])
-    actual_network_alternative = Network(
+             M2,
+             S2,
+             S1,
+             S3],
+            recycle={recycle, inner_recycle})])
+    alternative_network = Network(
         [P1,
          P2,
          P3,
          Network(
-            [Network(
-                [M1,
-                 M2,
-                 S2,
-                 S1],
-                recycle=S1-1),
-             S3],
-            recycle=S3-1)])
-    assert network == actual_network or network == actual_network_alternative
+            [M1,
+             M2,
+             S2,
+             S3,
+             S1],
+            recycle={inner_recycle, recycle})])
+    assert network == actual_network or network == alternative_network
     recycle_loop_sys.simulate()
     x_nested_solution = np.vstack([recycle.mol, inner_recycle.mol])
     recycle_loop_sys.flatten()
-    assert recycle_loop_sys.path == (P1, P2, P3, M1, M2, S2, S3, S1)
+    assert recycle_loop_sys.path == (P1, P2, P3, M1, M2, S2, S1, S3) or recycle_loop_sys.path == (P1, P2, P3, M1, M2, S2, S3, S1)
     recycle_loop_sys.empty_recycles()
     recycle_loop_sys.simulate()
     x_flat_solution = np.vstack([recycle.mol, inner_recycle.mol])
@@ -474,10 +489,11 @@ def test_feed_forward_recycle_loop():
                  S1,
                  M2,
                  S2],
-                recycle=[S2-0, S1-1]),
+                recycle=M1-0),
              P3])
     )
     assert network == actual_network 
+    recycle_loop_sys.set_tolerance(mol=1e-6, rmol=1e-6, subsystems=True)
     recycle_loop_sys.simulate()
     x_nested_solution = np.vstack([recycle.mol, inner_recycle.mol])
     recycle_loop_sys.flatten()
@@ -585,30 +601,25 @@ def test_nested_recycle_loops():
          Network(
             [M1,
              P2,
-             Network(
-                [H1,
-                 M3],
-                recycle=M3-0),
+             H1,
+             M3,
+             H1,
              P4,
-             Network(
-                [M4,
-                 M5,
-                 Network(
-                    [H2,
-                     M6,
-                     S1],
-                    recycle=S1-0),
-                 S2,
-                 P8,
-                 S3,
-                 Network(
-                    [M7,
-                     S4,
-                     H3],
-                    recycle=H3-0)],
-                recycle=H3-1),
+             M4,
+             M5,
+             H2,
+             M6,
+             S1,
+             H2,
+             S2,
+             H3,
+             P8,
+             S3,
+             M7,
+             S4,
+             H3,
              S5],
-            recycle=S5-1),
+            recycle={S5-1, S4-1}),
          M8])
     assert network == actual_network 
     x_nested_solution = np.vstack([i.mol for i in recycles])
@@ -692,6 +703,7 @@ if __name__ == '__main__':
     test_linear_case()
     test_unconnected_case()
     test_simple_recycle_loop()
+    test_single_unit_recycle_loop()
     test_unconnected_recycle_loop()
     test_unconnected_recycle_loops()
     test_inner_recycle_loop()
